import logging


class AddContextualFieldFilter(logging.Filter):
    def __init__(self, field_name: str, field_value: str) -> None:
        self._field_name: str = field_name
        self._field_value: str = field_value

    # SerivceLogRecord: Union[logging.LogRecord, str]
    def filter(self, record: logging.LogRecord) -> bool:
        record.__dict__[self._field_name] = self._field_value
        return True


class MaskPasswordsInLogs(logging.Filter):
    def filter(self, record: logging.LogRecord) -> bool:
        if record.__dict__.get('password', None) is not None:
            record.__dict__['password'] = "******"
        return True



from dataclasses import dataclass
from typing import List
from shapely.geometry import Point, LineString


@dataclass
class Location:
    def __init__(self, lat: float, lng: float):
        assert -90 <= lat <= 90
        assert -180 <= lng <= 180
        self.lat = lat
        self.lng = lng

    def as_point(self):
        return Point(self.lng, self.lat)


@dataclass
class Path:
    def __init__(self, locations: List[Location]):
        assert len(locations) > 2
        points = [location.as_point() for location in locations]
        self._path = LineString([[p.x, p.y] for p in points])

    def as_list(self):
        return [Location(lat=y, lng=x) for x, y in self._path.coords]



from dataclasses import dataclass
from datetime import datetime
from typing import List, Any, Dict
from uuid import UUID


@dataclass
class Cursor:
    id: UUID
    created: datetime


CursorJson = Dict[str, Any]


def mk_cursor(page_size: int, data: List[Any], total=None) -> CursorJson:
    cursor = {'page_size': page_size}
    if total:
        cursor['total'] = total

    if data:
        last = data[-1]
        last_string = f'{last.created_at.isoformat()}|{last.id}'
        cursor['last'] = last_string

    return cursor


def after_to_offset(after) -> Cursor:
    offset = after.split('|')
    dt = datetime.fromisoformat(offset[0])
    id = UUID(offset[1])
    return Cursor(id=id, created=dt)



import pytest

from shuttlis.utils import one_or_none, one


@pytest.mark.parametrize('lizt,key,expected', [
    ([1, 2, 3], lambda x: x == 2, 2),
    (['a', 'b', 'c'], lambda x: x == 'c', 'c'),
    (['a', 'b', 'c'], lambda x: x == 'd', None),
])
def test_one_or_none(lizt, key, expected):
    actual = one_or_none(lizt, key)
    assert expected == actual


def test_one_or_none_raises_assertion_error_if_more_than_one_match():
    with pytest.raises(AssertionError):
        one_or_none([1, 2, 2, 2], lambda x: x == 2)


@pytest.mark.parametrize('lizt,key,expected', [
    ([1, 2, 3], lambda x: x == 2, 2),
    (['a', 'b', 'c'], lambda x: x == 'c', 'c'),
])
def test_one(lizt, key, expected):
    actual = one(lizt, key)
    assert expected == actual


def test_one_raises_assertion_error_if_more_than_one_match():
    with pytest.raises(AssertionError):
        one([1, 2, 2, 2], lambda x: x == 2)


def test_one_raises_assertion_error_if_none_match():
    with pytest.raises(AssertionError):
        one([1, 2, 2, 2], lambda x: x == 4)



from uuid import uuid4

from shuttlis.serialization import serialize


def test_serialize_with_None():
    res = serialize(None)
    assert res is None


def test_serialize_with_uuid():
    uuid = uuid4()
    res = serialize(uuid)
    assert str(uuid) == res


def test_serialize_with_byte_array():
    bytes = b'Hey Dude!!!!'
    res = serialize(bytes)
    assert 'Hey Dude!!!!' == res



import pytest

from shuttlis.geography import Location


@pytest.mark.parametrize('lat,lng', [
    (90.0, 180.0),
    (-90.0, -180.0),
])
def test_location_constructer_with_valid_values(lat, lng):
    loc = Location(lat, lng)
    assert lat == loc.lat
    assert lng == loc.lng


@pytest.mark.parametrize('lat,lng', [
    (90, 190),
    (90.1, 179),
    (90.1, 180.001),
])
def test_location_constructer_disallows_lat_lng_values_which_are_out_of_range(lat, lng):
    with pytest.raises(AssertionError):
        Location(lat, lng)



import pytest
from pytz import timezone
from datetime import datetime

from shuttlis.time import MilitaryTime, TimeDelta, WeekDay, TimezoneMismatch

IST_TIMEZONE = timezone('Asia/Kolkata')


def _mt_ist(time):
    return MilitaryTime(time, IST_TIMEZONE)


@pytest.mark.parametrize('hr,min', [(1, 20), (23, 59), (0, 0)])
def test_military_time_constructor(hr, min):
    mt = MilitaryTime.from_hr_min(hr, min, IST_TIMEZONE)
    assert hr == mt.hr
    assert min == mt.min
    assert IST_TIMEZONE.zone == mt.tz.zone


@pytest.mark.parametrize('hr,min', [(24, 00), (12, 61)])
def test_military_time_constructor_fails_when_hr_or_min_are_out_of_range(hr, min):
    with pytest.raises(AssertionError):
        MilitaryTime.from_hr_min(hr, min, IST_TIMEZONE)


@pytest.mark.parametrize('mta,mtr', [
    (_mt_ist(1320) + TimeDelta(1, 20), _mt_ist(1440)),
    (_mt_ist(1320) + TimeDelta(0, 40), _mt_ist(1400)),
])
def test_military_time_add(mta, mtr):
    assert mtr == mta


@pytest.mark.parametrize('mts,mtr', [
    (_mt_ist(1320) - TimeDelta(1, 20), _mt_ist(1200)),
    (_mt_ist(1320) - TimeDelta(0, 40), _mt_ist(1240)),
])
def test_military_time_sub(mts, mtr):
    assert mtr == mts


def test_military_time_inequality():
    assert _mt_ist(1000) != _mt_ist(900)


def test_weekday_eq():
    assert WeekDay.MONDAY == WeekDay.MONDAY


@pytest.mark.parametrize('first,snd', [
    (WeekDay.MONDAY, WeekDay.TUESDAY),
    (WeekDay.MONDAY, WeekDay.SUNDAY),
])
def test_weekday_lt(first, snd):
    assert first < snd


def test_weekday_extract_from_datetime():
    dt = datetime(year=2019, month=1, day=1)
    assert WeekDay.TUESDAY == WeekDay.extract_from_datetime(dt)


def test_military_time_with_timezone():
    assert _mt_ist(900) != MilitaryTime(900, timezone("America/Sao_Paulo"))


def test_military_time_sort():
    source_times = [_mt_ist(r) for r in range(2300, 1, -100)]
    assert sorted(source_times) == [_mt_ist(r) for r in range(100, 2400, 100)]


def test_military_time_compare_lt():
    assert _mt_ist(100) < _mt_ist(150)


def test_military_time_compare_gt():
    assert _mt_ist(1000) > _mt_ist(150)


def test_military_time_compare_gte():
    assert _mt_ist(150) >= _mt_ist(150)
    assert _mt_ist(1000) >= _mt_ist(150)


def test_military_time_compare_lte():
    assert _mt_ist(150) <= _mt_ist(150)
    assert _mt_ist(100) <= _mt_ist(150)


def test_military_time_compare_timezone_mismatch():
    with pytest.raises(TimezoneMismatch):
        _mt_ist(100) < MilitaryTime(100, timezone("America/Sao_Paulo"))


def test_weekday_str_conversion():
    assert str(WeekDay.SUNDAY) == 'SUNDAY'


def test_time_delta_with_only_minutes():
    assert _mt_ist(1359) == _mt_ist(1300) + TimeDelta(min=59)


def test_hash_of_military_times():
    assert 2 == len({_mt_ist(900), _mt_ist(1000)})


def test_hash_of_same_military_time():
    assert 1 == len({_mt_ist(900), _mt_ist(900)})


def test_mt_in_list():
    assert _mt_ist(900) in [_mt_ist(900), _mt_ist(1000)]


def test_mt_not_in_list():
    assert _mt_ist(910) not in [_mt_ist(900), _mt_ist(1000)]



from setuptools import setup, find_packages

setup(
    name='pyshuttlis',
    version='0.1.6',
    description='Utilities',
    url='https://github.com/shuttl-tech/pyshuttlis',
    author='Shuttl',
    author_email='sherub.thakur@shuttl.com',
    license='MIT',
    packages=find_packages(),
    classifiers=["Programming Language :: Python :: 3.7"],
    install_requires=['shapely', 'pytz'],
    extras_require={
        'test': ['pytest', 'pytest-runner', 'pytest-cov', 'pytest-pep8'],
        'dev': ['flake8'],
    },
)



import logging
import sys
import json
import traceback
from .log_filters import AddContextualFieldFilter, MaskPasswordsInLogs


class JSONFormatter(logging.Formatter):
    def format(self, record: logging.LogRecord) -> str:
        json_dict = {}

        if record.exc_info:
            json_dict['stack_trace'] = _sanitize_stacktrace_for_json_fields(*record.exc_info)

            # mark exception info as none to consume the exception here and stop propogation
            # where it might get logged to sys.stderr.
            # If we need to reraise it we can log it as an exception on the root logger
            record.exc_info = None

        json_dict.update(record.__dict__)

        # Already replaced this with 'stack_trace', no longer needed
        assert json_dict['exc_info'] is None
        del json_dict['exc_info']

        # We don't need these 2 fields as message field has msg % args now
        json_dict['message'] = record.getMessage()
        del json_dict['msg']
        del json_dict['args']

        json_dict['level'] = json_dict['levelname']
        del json_dict['levelname']

        try:
            return json.dumps(json_dict)
        except (TypeError, OverflowError):
            return json.dumps({
                "message":
                "An unrecoverable exception occured while logging"
            })


def _sanitize_stacktrace_for_json_fields(type, value, traceback) -> str:
    lines = traceback.format_exception(type, value, traceback)
    return "".join(lines)


def _uncaught_exception_logger(
    exception_type: BaseException,
    exception_value: Exception,
    exception_traceback: traceback.TracebackException
):
    exception_traceback_string = _sanitize_stacktrace_for_json_fields(
        exception_type,
        exception_value,
        exception_traceback,
    )
    # raise to root logger where it will be consumed by the formatter attached to our handler
    logging.getLogger().error(exception_traceback_string)


_default_formatter = logging.Formatter(
        '[%(levelname)s -- %(asctime)s] :: (%(module)s.%(funcName)s) :: %(message)s',
        "%H:%M:%S"
    )


def configure_logging(_logger_name: str, level: str, log_format: str = 'json') -> logging.Logger:
    logger = logging.getLogger()
    logger.addHandler(logging.StreamHandler())
    logger.setLevel(level)

    assert logger.hasHandlers()
    for handler in logger.handlers:
        if log_format != 'console':
            handler.setFormatter(JSONFormatter())
        else:
            handler.setFormatter(_default_formatter)
        handler.addFilter(MaskPasswordsInLogs())
        handler.addFilter(AddContextualFieldFilter('service', 'rms'))

    sys.excepthook = _uncaught_exception_logger

    return logging.getLogger(_logger_name)



import calendar
from datetime import datetime
from enum import Enum
from functools import total_ordering


@total_ordering
class WeekDay(Enum):
    MONDAY = 'MONDAY'
    TUESDAY = 'TUESDAY'
    WEDNESDAY = 'WEDNESDAY'
    THURSDAY = 'THURSDAY'
    FRIDAY = 'FRIDAY'
    SATURDAY = 'SATURDAY'
    SUNDAY = 'SUNDAY'

    @classmethod
    def extract_from_datetime(cls, dt: datetime):
        return cls[calendar.day_name[dt.weekday()].upper()]

    @classmethod
    def from_string(cls, weekday: str):
        return cls[weekday.upper()]

    def __str__(self):
        return str(self.name)

    def __lt__(self, other):
        return _order.index(self) < _order.index(other)


_order = [
    WeekDay.MONDAY,
    WeekDay.TUESDAY,
    WeekDay.WEDNESDAY,
    WeekDay.THURSDAY,
    WeekDay.FRIDAY,
    WeekDay.SATURDAY,
    WeekDay.SUNDAY,
]



import itertools as it

from typing import List, Callable, Any, Optional, Dict


def one_or_none(lizt: List[Any], key: Callable[[Any], bool]) -> Optional[Any]:
    gen = (x for x in lizt if key(x))
    elm = next(gen, None)
    if not None:
        assert next(gen, None) is None
    return elm


def one(lizt: List[Any], key: Callable[[Any], bool]) -> Any:
    maybe_elm = one_or_none(lizt, key)
    assert maybe_elm is not None
    return maybe_elm


def group_by(lizt: List[Any], key: Callable[[Any], Any]) -> Dict[Any, List[Any]]:
    sorted_list = sorted(lizt, key=key)
    return {k: list(v) for k, v in it.groupby(sorted_list, key)}
